from pathlib import Path
from typing import Dict, List
from uuid import uuid4

from excel_provider.io.excel import read_excel


class RestHandler:
    def __init__(self, handler_config: Dict = None):
        self.data: Dict[str, Dict] = None
        self.sheets: Dict[str, str] = None
        self.config = handler_config

    def read_data(self):
        if self.config is None:
            raise ValueError("Handler config is not set")

        if not self.config_valid():
            raise ValueError("Handler config is not valid")

        read_config = {
            "file": Path(self.config["excel_file"]),
            "sheets": self.config["sheets"],
            "data_cols": self.config["data_cols"],
            "index_cols": self.config.get("index_cols"),
        }

        self.data = read_excel(**read_config)
        self.sheets = [
            {"id": str(uuid4()), "name": sheet} for sheet in self.data.keys()
        ]

    def config_valid(self) -> bool:
        return (
            self.config is not None
            and self.config.get("excel_file") is not None
            and isinstance(self.config.get("sheets"), list)
            and isinstance(self.config.get("data_cols"), list)
        )

    def get_data(self, sheet_id: str) -> Dict:
        if self.sheets is None or self.data is None:
            raise ValueError("Data has not been read")

        if sheet := self._get_sheet_name_by_id(sheet_id):
            return {
                "id": sheet_id,
                "name": sheet,
                "series": [
                    {"name": name, "rows": rows}
                    for name, rows in self.data[sheet].items()
                ],
            }
        else:
            raise ValueError(f"Sheet with id {sheet_id} does not exist")

    def _get_sheet_name_by_id(self, sheet_id: str) -> str:
        for id, name in self.sheets.items():
            if id == sheet_id:
                return name
        return None

    def get_sheet_names(self) -> Dict[str, str]:
        if self.sheets is None:
            raise ValueError("Data has not been read")

        return {"sheets": self.sheets}



from typing import Dict

from flask import Flask, jsonify
from flask_swagger import swagger

from excel_provider.rest.handler import RestHandler


def create_app(config: Dict):
    app = Flask(__name__)

    handler = RestHandler(config.get("handler"))
    handler.read_data()

    setattr(app, "handler", handler)

    @app.route("/sheets", methods=["GET"])
    def get_sheets():
        """
        Get all sheets
        Returns a list of all sheets with their name and ID
        ---
        definitions:
          - schema:
              id: SheetShort
              type: object
              properties:
                id:
                  type: string
                name:
                  type: string
              required:
                - id
                - name
          - schema:
              id: SheetsOverview
              type: object
              properties:
                sheets:
                  type: array
                  items:
                    $ref: '#/definitions/SheetShort'
              required:
                - sheets
        responses:
          200:
            description: A list of all sheets
            schema:
              $ref: '#/definitions/SheetsOverview'
        """
        return jsonify(app.handler.get_sheet_names())

    @app.route("/sheet/<id>", methods=["GET"])
    def get_sheet(id):
        """
        Get detail of a sheet
        Returns the data of a sheet
        ---
        definitions:
          - schema:
              id: SheetDetail
              type: object
              properties:
                id:
                  type: string
                name:
                  type: string
                series:
                  type: array
                  items:
                    $ref: '#/definitions/Series'
              required:
                - id
                - name
                - series
          - schema:
              id: Series
              type: object
              properties:
                name:
                  type: string
                rows:
                  type: object
                  additionalProperties:
                    type: string
              required:
                - name
                - rows
        parameters:
          - name: id
            in: path
            type: string
            required: true
            description: The ID of the sheet
        responses:
          200:
            description: The data of the sheet
            schema:
              $ref: '#/definitions/SheetDetail'
          404:
            description: Sheet not found
            schema:
              type: object
              properties:
                error:
                  type: string

        """
        try:
            return jsonify(app.handler.get_data(id))
        except ValueError as e:
            return {"error": str(e)}, 404

    @app.route("/refresh", methods=["POST"])
    def post_refresh():
        """
        Refresh the data
        Reads the data from the Excel file again
        ---
        responses:
          200:
            description: Data refreshed
        """
        app.handler.read_data()
        return jsonify({"message": "Data refreshed"})

    @app.route("/spec")
    def spec():
        swag = swagger(app)
        swag["info"]["version"] = "1.0"
        swag["info"]["title"] = "Excel Provider"
        return jsonify(swag)

    return app
